@@ -1,8 +1,11 @@
 /*
+** fs_types
 ** The Sleuth Kit 
 **
+** $Date: 2005/01/17 22:40:16 $
+**
 ** Brian Carrier [carrier@sleuthkit.org]
-** Copyright (c) 2003 Brian Carrier.  All rights reserved
+** Copyright (c) 2003-2005 Brian Carrier.  All rights reserved
 **
 ** TASK
 ** Copyright (c) 2002 @stake Inc.  All rights reserved
@@ -13,6 +16,8 @@
 
 extern char fs_parse_type(const char *);
 extern void fs_print_types();
+extern char *fs_get_type(char);
+extern void print_version();
 
 /*
 ** the most-sig-nibble is the file system type, which indictates which
@@ -22,7 +27,7 @@ extern void fs_print_types();
 #define FSMASK			0xf0
 #define OSMASK			0x0f
 
-#define UNSUPP_FS       0x00
+#define UNSUPP_FS		0x00
 
 /* FFS */
 #define FFS_TYPE		0x10
@@ -30,23 +35,24 @@ extern void fs_print_types();
 #define FFS_1			0x11	/* UFS1 - FreeBSD, OpenBSD, BSDI ... */
 #define FFS_1B			0x12	/* Solaris (no type) */
 #define FFS_2			0x13	/* UFS2 - FreeBSD, NetBSD */
+#define FFSAUTO			0x14
 
 
 #define	EXTxFS_TYPE		0x20
-#define EXT2FS			0x21	
-#define EXT3FS			0x22	
+#define EXT2FS			0x21
+#define EXT3FS			0x22
 #define EXTAUTO			0x23
 
 /* FAT */
 #define FATFS_TYPE		0x30
 
-#define MS12_FAT        0x31
-#define MS16_FAT        0x32
-#define MS32_FAT        0x33
-#define MSAUTO_FAT		0x34
+#define FAT12		0x31
+#define FAT16		0x32
+#define FAT32		0x33
+#define FATAUTO		0x34
 
 #define NTFS_TYPE		0x40
-#define NTFS         0x40
+#define NTFS			0x40
 
 #define	SWAPFS_TYPE		0x50
 #define	SWAP			0x50
