@@ -29,17 +29,19 @@
 
 #define MD5_HASH_LENGTH	16
 
-int main(argc, argv)
-int     argc;
-char  **argv;
+int
+main(argc, argv)
+  int argc;
+  char **argv;
 {
-    char   *myname = argv[0];
-    char   *crunch();
-    FILE   *fp;
+    char *myname = argv[0];
+    char *crunch();
+    FILE *fp;
 
     if (argc < 2) {
 	printf("%s\n", crunch(stdin));
-    } else {
+    }
+    else {
 	while (--argc && *++argv) {
 	    if ((fp = fopen(*argv, "r")) == 0) {
 		fprintf(stderr, "%s: ", myname);
@@ -53,16 +55,17 @@ char  **argv;
     return (0);
 }
 
-char   *crunch(fp)
-FILE   *fp;
+char *
+crunch(fp)
+  FILE *fp;
 {
     MD5_CTX md;
     unsigned char sum[MD5_HASH_LENGTH];
     unsigned char buf[BUFSIZ];
     static char result[2 * MD5_HASH_LENGTH + 1];
     static char hex[] = "0123456789abcdef";
-    int     buflen;
-    int     i;
+    int buflen;
+    int i;
 
     MD5Init(&md);
     while ((buflen = fread(buf, 1, BUFSIZ, fp)) > 0)
