@@ -49,9 +49,10 @@
 
 /* split_at - break string at first delimiter, return remainder */
 
-char   *split_at(char *string, int delimiter)
+char *
+split_at(char *string, int delimiter)
 {
-    char   *cp;
+    char *cp;
 
     if ((cp = strchr(string, delimiter)) != 0)
 	*cp++ = 0;
@@ -60,9 +61,10 @@ char   *split_at(char *string, int delimiter)
 
 /* split_at_right - break string at last delimiter, return remainder */
 
-char   *split_at_right(char *string, int delimiter)
+char *
+split_at_right(char *string, int delimiter)
 {
-    char   *cp;
+    char *cp;
 
     if ((cp = strrchr(string, delimiter)) != 0)
 	*cp++ = 0;
