@@ -1,8 +1,10 @@
-/*
+/* 
  * The Sleuth Kit
  *
+ * $Date: 2005/01/17 22:40:17 $
+ *
  * Brian Carrier [carrier@sleuthkit.org]
- * Copyright (c) 2003 Brian Carrier.  All rights reserved
+ * Copyright (c) 2003-2005 Brian Carrier.  All rights reserved
  *
  * This file is part of mmtools
  *
@@ -16,19 +18,25 @@
 #define _DOS_H
 
 typedef struct {
-    u_int8_t boot;
-    u_int8_t start_chs[3];
-    u_int8_t ptype;
-    u_int8_t end_chs[3];
-    u_int8_t start_sec[4];
-    u_int8_t size_sec[4];
+    uint8_t boot;
+    uint8_t start_chs[3];
+    uint8_t ptype;
+    uint8_t end_chs[3];
+    uint8_t start_sec[4];
+    uint8_t size_sec[4];
 } dos_part;
 
 /* Boot Sector w/partition table */
 typedef struct {
-    u_int8_t filler[446];
+    uint8_t f1[3];
+    /* the next three are actually part of NTFS and FAT, but
+     * we use them for sanity checks in the detect code */
+    char oemname[8];
+    uint8_t ssize[2];		/* sector size in bytes */
+    uint8_t csize;		/* cluster size in sectors */
+    uint8_t filler[432];
     dos_part ptable[4];
-    u_int8_t magic[2];
+    uint8_t magic[2];
 } dos_sect;
 
 #define DOS_MAGIC	0xaa55
