@@ -1,8 +1,10 @@
 /*
  * The Sleuth Kit
  *
+ * $Date: 2005/01/17 22:40:17 $
+ *
  * Brian Carrier [carrier@sleuthkit.org]
- * Copyright (c) 2003 Brian Carrier.  All rights reserved
+ * Copyright (c) 2003-2005 Brian Carrier.  All rights reserved
  *
  * This file is part of mmtools
  *
@@ -17,25 +19,25 @@
 #define _MAC_H
 
 typedef struct {
-    u_int8_t magic[2];
-    u_int8_t reserved[2];
-	u_int8_t pmap_size[4];
-	u_int8_t start_sec[4];
-	u_int8_t size_sec[4];
-	u_int8_t name[32];
-	u_int8_t type[32];
-	u_int8_t data_start_sec[4];
-	u_int8_t data_size_sec[4];
-	u_int8_t status[4];
-	u_int8_t boot_start_sec[4];
-	u_int8_t boot_size_sec[4];
-	u_int8_t boot_load_addr[4];
-	u_int8_t reserved2[4];
-	u_int8_t boot_entry[4];
-	u_int8_t reserved3[4];
-	u_int8_t boot_checksum[4];
-	u_int8_t proc_type[16];
-	u_int8_t reserved4[376];
+    uint8_t magic[2];
+    uint8_t reserved[2];
+    uint8_t pmap_size[4];
+    uint8_t start_sec[4];
+    uint8_t size_sec[4];
+    uint8_t name[32];
+    uint8_t type[32];
+    uint8_t data_start_sec[4];
+    uint8_t data_size_sec[4];
+    uint8_t status[4];
+    uint8_t boot_start_sec[4];
+    uint8_t boot_size_sec[4];
+    uint8_t boot_load_addr[4];
+    uint8_t reserved2[4];
+    uint8_t boot_entry[4];
+    uint8_t reserved3[4];
+    uint8_t boot_checksum[4];
+    uint8_t proc_type[16];
+    uint8_t reserved4[376];
 } mac_part;
 
 #define MAC_MAGIC	0x504d
